<?php
class Photo extends AppModel {
	var $name = 'Photo';

	var $hasAndBelongsToMany = array (
										'Tag' => array (
										'className' => 'Tag',
										'order' => 'tag')
									);

	var $hasMany = array (
			'Comment' => array (
			'className' => 'Comment',
			'foreignKey' => 'photo_id',
			'order' => 'Comment.created DESC',
			'dependent' => true,
			'exclusive' => false
		)
	);

	var $validate = array (
		'title' => VALID_NOT_EMPTY,
		'filename' => VALID_NOT_EMPTY
	);

	function savePhoto($fileData,$thumbSize,$thumbType) {
		vendor('ccImageResize.class');
		if (move_uploaded_file($fileData['tmp_name'], WWW_ROOT . "img/photos/" . $fileData['name'])) {
			$resizer = new ccImageResize;
			if ($resizer->resizeImage(WWW_ROOT . "img/photos/" . $fileData['name'], WWW_ROOT . "img/thumbnails/" . $fileData['name'],$thumbSize,$thumbType)) {
				return true;
			} else {
				// TODO: Clean up and set error message
				return false;
			}
		} else {
			// TODO: Set error message if move_uploaded_file fails
			return false;
		}
	}

	function delPhoto($filename) {
		/**
		 * TODO: Code for condition where unlink fails
		 */
		unlink(WWW_ROOT . "img/photos/" . $filename);
		unlink(WWW_ROOT . "img/thumbnails/" . $filename);
		return true;
	}

	/**
	 * filmstrip is similar to the findNeighbours method but returns an array of previous and next items
	 * taking care about and adjusting forthe begining and end of series
	 * @param integer $id - id of the current photo
	 * @param integer $count - Count of total items needed in the filmstrip
	 * @return array
	 */
	function filmstrip($id, $count = 5) {

		// Calculate how many next and prev
		$limit = (int) ($count / 2);

		// Get the next pictures
		$next = $this->findAll("id > $id", array (
													'id',
													'filename',
													'title'
												 ), " id ASC", $limit, null, 0);
		$nextCount = count($next);

		// If there were not enough of next pictures add the difference to limit for getting prev pictures
		if ($nextCount < $limit) {
			$limit += ($limit - $nextCount);
		}

		// Get the previous pictures
		$prev = $this->findAll("id < $id", array (
													'id',
													'filename',
													'title'
												 ), " id DESC", $limit, null, 0);

		$prevCount = count($prev);

		// Not enough prev pictures got but the next were enough chances are that there are more next pictures
		// try getting more next pictures
		if ($prevCount < $limit && $nextCount == $limit) {
			$limit += ($limit - $prevCount);
			$next = $this->findAll("id > $id", array (
														'id',
														'filename',
														'title'
													 ), " id ASC", $limit, null, 0);
		}

		//TODO: Find a way to get the current item into the return array
		$current = array();

		return array (
			'next' => $next,
			'current' => $current,
			'prev' => $prev
		);
	}

	/**
	 * Return the count of photos for a given tag
	 */
	function findCountByTag ($tag) {
		if (empty($tag)){
			return 0;
		}
		$prefix = $this->tablePrefix;
		$count = $this->query("SELECT count(Photo.id) AS count
									FROM {$prefix}photos Photo, {$prefix}photos_tags photos_tags, {$prefix}tags tags
									WHERE Photo.id = photos_tags.photo_id
										  AND tags.id = photos_tags.tag_id
										  AND tags.tag ='$tag'");
		return $count[0][0]['count'];
	}

	function findAllByTag ($tag, $limit=50,$page=1) {
		if (empty($tag)){
			return false;
		}
		$prefix = $this->tablePrefix;
		$offset = $limit * ($page-1);
        $photos = $this->query("SELECT Photo.id,Photo.filename,Photo.title
											FROM {$prefix}photos Photo, {$prefix}photos_tags photos_tags, {$prefix}tags tags
											WHERE Photo.id = photos_tags.photo_id
												  AND tags.id = photos_tags.tag_id
												  AND tags.tag ='$tag'
												  ORDER BY Photo.created DESC
												  LIMIT $offset,$limit");
		return $photos;
	}
}
?>